#include "operationdemo1.hpp"
#include "operationconnector.hpp"
#include "itemtypes.hpp"

#include <qschematic/items/label.hpp>

struct ConnectorAttribute {
    QPoint point;
    QString name;
};

OperationDemo1::OperationDemo1(QGraphicsItem* parent) :
    Operation(::ItemType::OperationDemo1Type, parent)
{
    setSize(160, 160);
    label()->setText(QStringLiteral("Demo 1"));

    QVector<ConnectorAttribute> connectorAttributes = {
        { QPoint(0, 2), QStringLiteral("in 1") },
        { QPoint(0, 4), QStringLiteral("in 2") },
        { QPoint(0, 6), QStringLiteral("in 3") },
        { QPoint(8, 4), QStringLiteral("out") }
    };

    for (const auto& c : connectorAttributes) {
        auto connector = std::make_shared<OperationConnector>(c.point, c.name);
        connector->label()->setVisible(true);
        addConnector(connector);
    }
}

gpds::container OperationDemo1::to_container() const
{
    // Root
    gpds::container root;
    addItemTypeIdToContainer(root);
    root.add_value("operation", Operation::to_container());

    return root;
}

void OperationDemo1::from_container(const gpds::container& container)
{
    // Root
    Operation::from_container(*container.get_value<gpds::container*>("operation").value());
}

std::shared_ptr<QSchematic::Items::Item> OperationDemo1::deepCopy() const
{
    auto clone = std::make_shared<OperationDemo1>(parentItem());
    copyAttributes(*clone);

    return clone;
}

void OperationDemo1::copyAttributes(OperationDemo1& dest) const
{
    Operation::copyAttributes(dest);
}
